/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.loadtest;

import com.eviware.soapui.impl.wsdl.loadtest.WsdlLoadTest;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.HttpSettings;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.support.AbstractSoapUIAction;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormField;
import com.eviware.x.form.XFormFieldListener;
import com.eviware.x.form.support.ADialogBuilder;
import com.eviware.x.form.support.AField;
import com.eviware.x.form.support.AForm;
import com.eviware.x.form.support.AField.AFieldType;

/**
 * Displays the LoadTest Options dialog
 * 
 * @author Ole.Matzura
 */

public class LoadTestOptionsAction extends AbstractSoapUIAction<WsdlLoadTest>
{
	public static final String SOAPUI_ACTION_ID = "LoadTestOptionsAction";
	private XFormDialog dialog;

	public LoadTestOptionsAction()
   {
		super( "Options","Sets options for this LoadTest" );
   }
	
	public void perform( WsdlLoadTest loadTest, Object param )
	{
		if( dialog == null )
			buildDialog();
		
		dialog.setIntValue( Form.THREAD_STARTUP_DELAY, loadTest.getStartDelay() );
		dialog.setBooleanValue( Form.RESET_STATISTICS, loadTest.getResetStatisticsOnThreadCountChange() );
		dialog.setBooleanValue( Form.CALC_TPS, loadTest.getCalculateTPSOnTimePassed() );
		dialog.setIntValue( Form.SAMPLE_INTERVAL, ( int ) loadTest.getSampleInterval() );
		dialog.setBooleanValue( Form.DISABLE_HISTORY, loadTest.getHistoryLimit() == 0 );
		dialog.setIntValue( Form.MAX_ASSERTIONS, ( int ) loadTest.getMaxAssertionErrors() );
		
		Settings settings = loadTest.getSettings();
		
		dialog.setBooleanValue( Form.INCLUDE_REQUEST, settings.getBoolean( HttpSettings.INCLUDE_REQUEST_IN_TIME_TAKEN ));
		dialog.setBooleanValue( Form.INCLUDE_RESPONSE, settings.getBoolean( HttpSettings.INCLUDE_RESPONSE_IN_TIME_TAKEN ));
		dialog.setBooleanValue( Form.CLOSE_CONNECTIONS, settings.getBoolean( HttpSettings.CLOSE_CONNECTIONS ));
	
		if( dialog.show() )
		{
			try
			{
				loadTest.setStartDelay( dialog.getIntValue( Form.THREAD_STARTUP_DELAY, loadTest.getStartDelay() ));
				loadTest.setResetStatisticsOnThreadCountChange( dialog.getBooleanValue( Form.RESET_STATISTICS ));
				loadTest.setCalculateTPSOnTimePassed( dialog.getBooleanValue( Form.CALC_TPS ));
				loadTest.setSampleInterval( dialog.getIntValue( Form.SAMPLE_INTERVAL, ( int ) loadTest.getSampleInterval() ));
				loadTest.setHistoryLimit( dialog.getBooleanValue( Form.DISABLE_HISTORY ) ? 0 : -1 );
				loadTest.setMaxAssertionErrors( dialog.getIntValue( Form.MAX_ASSERTIONS, 1000 ));
				
				
				settings.setBoolean( HttpSettings.INCLUDE_REQUEST_IN_TIME_TAKEN, 
							dialog.getBooleanValue( Form.INCLUDE_REQUEST ));
				settings.setBoolean( HttpSettings.INCLUDE_RESPONSE_IN_TIME_TAKEN, 
							dialog.getBooleanValue( Form.INCLUDE_RESPONSE ));
				settings.setBoolean( HttpSettings.CLOSE_CONNECTIONS, 
							dialog.getBooleanValue( Form.CLOSE_CONNECTIONS ));
			}
			catch (NumberFormatException ex)
			{
			}			
		}
	}

	private void buildDialog()
	{
	   dialog = ADialogBuilder.buildDialog( Form.class );
	   dialog.getFormField( Form.DISABLE_HISTORY ).addFormFieldListener( new XFormFieldListener() {

			public void valueChanged( XFormField sourceField, String newValue, String oldValue )
			{
				dialog.getFormField( Form.SAMPLE_INTERVAL ).setEnabled( !Boolean.parseBoolean( newValue ) );
			}} );
	}
	
	@AForm( name="LoadTest Options", description = "Set options for this LoadTest", 
				helpUrl=HelpUrls.LOADTESTOPTIONS_HELP_URL, icon=UISupport.OPTIONS_ICON_PATH )
	private interface Form 
	{
		@AField(name = "Thread startup delay", description = "The delay before starting a thread in ms", type = AFieldType.INT )
		public final static String THREAD_STARTUP_DELAY = "Thread startup delay";
		
		@AField(name = "Reset statistics", description = "when the number of threads changes", type = AFieldType.BOOLEAN )
		public final static String RESET_STATISTICS = "Reset statistics";

		@AField(name = "Calculate TPS/BPS", description = "based on actual time passed", type = AFieldType.BOOLEAN )
		public final static String CALC_TPS = "Calculate TPS/BPS";
		
		@AField(name = "Include request write", description = "in calculated time", type = AFieldType.BOOLEAN )
		public final static String INCLUDE_REQUEST = "Include request write";
		
		@AField(name = "Include response read", description = "in calculated time", type = AFieldType.BOOLEAN )
		public final static String INCLUDE_RESPONSE = "Include response read";
		
		@AField(name = "Close connections", description = "between each request", type = AFieldType.BOOLEAN )
		public final static String CLOSE_CONNECTIONS = "Close connections";
		
		@AField(name = "Sample interval", description = "in calculated time", type = AFieldType.INT )
		public final static String SAMPLE_INTERVAL = "Sample interval";
		
		@AField(name = "Disable History", description = "to preserve memory (will disable diagrams)", type = AFieldType.BOOLEAN )
		public final static String DISABLE_HISTORY = "Disable History";
		
		@AField(name = "Max Assertions in Log", description = "the maximum number of assertion errors to keep in log (to preserve memory)", type = AFieldType.INT )
		public final static String MAX_ASSERTIONS = "Max Assertions in Log";
	}
}