/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.endpoint;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;

import org.apache.log4j.Logger;
import org.jdesktop.swingx.JXTable;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.actions.iface.InterfaceEndpointsAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.endpoint.DefaultEndpointStrategy.EndpointDefaults;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.support.AbstractSoapUIAction;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * Manages the service endpoints for a WsdlInterface
 * 
 * @author Ole.Matzura
 */

public class DefaultEndpointStrategyConfigureAction extends AbstractSoapUIAction<WsdlInterface>
{
	public static final String SOAPUI_ACTION_ID = "DefaultEndpointStrategyConfigureAction";

	private JDialog dialog;
	private JXTable table;
	private EndpointsTableModel tableModel;
	private WsdlInterface iface;
	private final static Logger log = Logger.getLogger( InterfaceEndpointsAction.class );
	private JButton deleteButton;
	private JButton assignButton;

	public DefaultEndpointStrategyConfigureAction()
	{
		super( "Service Endpoints", "Manage service endpoints available for this interface" );
	}

	private void buildDialog()
	{
		dialog = new JDialog( UISupport.getMainFrame() );
		dialog.setTitle( "Interface Service Endpoints" );

		JPanel contentPanel = new JPanel( new BorderLayout() );
		tableModel = new EndpointsTableModel();
		table = new JXTable( tableModel );
		table.setHorizontalScrollEnabled( true );
		table.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		table.getSelectionModel().addListSelectionListener( new ListSelectionListener()
		{

			public void valueChanged( ListSelectionEvent e )
			{
				enableButtons();
			}
		} );
		table.getColumn( 0 ).setPreferredWidth( 250 );
		table.getColumn( 4 ).setCellEditor( new DefaultCellEditor(
					new JComboBox( new String[] {WsdlRequest.PW_TYPE_NONE, WsdlRequest.PW_TYPE_TEXT, WsdlRequest.PW_TYPE_DIGEST})) );

		JScrollPane scrollPane = new JScrollPane( table );

		scrollPane.setBorder( BorderFactory.createCompoundBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
					BorderFactory.createLineBorder( Color.GRAY ) ) );

		scrollPane.setVerticalScrollBarPolicy( ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS );

		contentPanel.add( scrollPane, BorderLayout.CENTER );
		contentPanel.add( createButtons(), BorderLayout.SOUTH );
		Component descriptionPanel = UISupport.buildDescription( "Service Endpoints",
					"Edit available service endpoints for this interface and their defaults in table below", UISupport.OPTIONS_ICON );
		contentPanel.add( descriptionPanel, BorderLayout.NORTH );

		dialog.setContentPane( contentPanel );
		dialog.setSize( 650, 300 );

		dialog.setModal( true );
	}

	protected void enableButtons()
	{
		deleteButton.setEnabled( table.getSelectedRow() != -1 );
		assignButton.setEnabled( table.getSelectedRow() != -1 );
	}

	public void perform( WsdlInterface iface, Object param )
	{
		if( dialog == null )
			buildDialog();

		tableModel.setInterface( iface, ( DefaultEndpointStrategy ) param );

		this.iface = iface;
		enableButtons();
		UISupport.showDialog( dialog );
	}

	private Component createButtons()
	{
		ButtonBarBuilder builder = ButtonBarBuilder.createLeftToRightBuilder();
		ShowOnlineHelpAction showOnlineHelpAction = new ShowOnlineHelpAction( HelpUrls.ENDPOINTSEDITOR_HELP_URL );
		builder.addFixed( UISupport.createToolbarButton( showOnlineHelpAction ) );
		builder.addGlue();
		builder.addFixed( new JButton( new AddAction() ) );
		builder.addRelatedGap();
		deleteButton = new JButton( new DeleteAction() );
		builder.addFixed( deleteButton );
		builder.addRelatedGap();
		assignButton = new JButton( new AssignAction() );
		builder.addFixed( assignButton );
		builder.addRelatedGap();
		JButton okButton = new JButton( new OkAction() );
		builder.addFixed( okButton );

		builder
					.setBorder( BorderFactory.createCompoundBorder( BorderFactory.createCompoundBorder( BorderFactory
								.createMatteBorder( 1, 0, 0, 0, Color.GRAY ), BorderFactory.createMatteBorder( 1, 0, 0, 0,
								Color.WHITE ) ), BorderFactory.createEmptyBorder( 3, 5, 3, 5 ) ) );

		UISupport.initDialogActions( dialog, showOnlineHelpAction, okButton );

		return builder.getPanel();
	}

	private class AddAction extends AbstractAction
	{
		public AddAction()
		{
			super( "Add" );
			putValue( Action.SHORT_DESCRIPTION, "Adds a new endpoint to the list" );
		}

		public void actionPerformed( ActionEvent e )
		{
			String endpoint = UISupport.prompt( "Enter new endpoint URL", "Add Endpoint", "" );

			if( endpoint == null )
				return;

			tableModel.addEndpoint( endpoint );
		}
	}

	private class AssignAction extends AbstractAction
	{
		private static final String ALL_REQUESTS = "- All Requests -";
		private static final String ALL_TEST_REQUESTS = "- All Test Requests -";
		private static final String ALL_REQUESTS_AND_TEST_REQUESTS = "- All Requests and TestRequests -";
		private static final String ALL_REQUESTS_WITH_NO_ENDPOINT = "- All Requests with no endpoint -";

		public AssignAction()
		{
			super( "Assign" );
			putValue( Action.SHORT_DESCRIPTION,
						"Assigns the selected endpoint to Requests/TestRequests for this Interface" );
		}

		public void actionPerformed( ActionEvent e )
		{
			int selectedIndex = table.getSelectedRow();
			if( selectedIndex == -1 )
			{
				Toolkit.getDefaultToolkit().beep();
				return;
			}

			String selectedEndpoint = ( String ) tableModel.getEndpointAt( selectedIndex );

			List<String> list = new ArrayList<String>( Arrays.asList( iface.getEndpoints() ) );
			list.add( 0, ALL_REQUESTS );
			list.add( 1, ALL_TEST_REQUESTS );
			list.add( 2, ALL_REQUESTS_AND_TEST_REQUESTS );
			list.add( 3, ALL_REQUESTS_WITH_NO_ENDPOINT );

			Object endpoint = UISupport.prompt( "Assign selected endpoint to..", "Assign Endpoint", list.toArray(),
						ALL_REQUESTS_WITH_NO_ENDPOINT );

			if( endpoint == null )
				return;

			int changeCount = 0;

			if( endpoint.equals( ALL_REQUESTS ) || endpoint.equals( ALL_REQUESTS_WITH_NO_ENDPOINT )
						|| endpoint.equals( ALL_REQUESTS_AND_TEST_REQUESTS ) )
			{
				for( int c = 0; c < iface.getOperationCount(); c++ )
				{
					Operation operation = iface.getOperationAt( c );
					for( int i = 0; i < operation.getRequestCount(); i++ )
					{
						Request request = operation.getRequestAt( i );
						String ep = request.getEndpoint();

						if( endpoint.equals( ALL_REQUESTS ) || endpoint.equals( ALL_REQUESTS_AND_TEST_REQUESTS )
									|| ( endpoint.equals( ALL_REQUESTS_WITH_NO_ENDPOINT ) && ep == null )
									|| ( ep.equals( endpoint ) ) )
						{
							request.setEndpoint( selectedEndpoint );
							changeCount++;
						}
					}
				}
			}

			if( endpoint.equals( ALL_REQUESTS_AND_TEST_REQUESTS ) || endpoint.equals( ALL_TEST_REQUESTS ) )
			{
				for( TestSuite testSuite : iface.getProject().getTestSuites() )
				{
					for( TestCase testCase : testSuite.getTestCaseList() )
					{
						for( TestStep testStep : testCase.getTestStepList() )
						{
							if( testStep instanceof WsdlTestRequestStep )
							{
								WsdlTestRequest testRequest = ( ( WsdlTestRequestStep ) testStep ).getTestRequest();
								testRequest.setEndpoint( selectedEndpoint );
								changeCount++;
							}
						}
					}
				}
			}

			log.info( "Assigned endpoint [" + selectedEndpoint + "] to " + changeCount + " Requests" );
		}
	}

	private class DeleteAction extends AbstractAction
	{
		public DeleteAction()
		{
			super( "Delete" );
			putValue( Action.SHORT_DESCRIPTION, "Deletes the selected endpoint from the list" );
		}

		public void actionPerformed( ActionEvent e )
		{
			int index = table.getSelectedRow();
			if( index == -1 )
			{
				Toolkit.getDefaultToolkit().beep();
				return;
			}

			if( UISupport.confirm( "Delete selected endpoint?", "Delete Endpoint" ) )
			{
				tableModel.removeEndpoint( index );
			}
		}
	}

	private class OkAction extends AbstractAction
	{
		public OkAction()
		{
			super( "OK" );
			putValue( Action.SHORT_DESCRIPTION, "Closes this dialog" );
		}

		public void actionPerformed( ActionEvent e )
		{
			dialog.setVisible( false );
		}
	}

	private class EndpointsTableModel extends AbstractTableModel
	{
		private WsdlInterface iface;
		private DefaultEndpointStrategy strategy;

		public int getColumnCount()
		{
			return 6;
		}

		@Override
		public String getColumnName( int column )
		{
			switch( column )
			{
			case 0:
				return "Endpoint";
			case 1:
				return "Username";
			case 2:
				return "Password";
			case 3:
				return "Domain";
			case 4:
				return "WSS-Type";
			case 5:
				return "WSS-TimeToLive";
			}

			return null;
		}

		public String getEndpointAt( int selectedIndex )
		{
			return iface.getEndpoints()[selectedIndex];
		}

		public void addEndpoint( String endpoint )
		{
			int rowCount = getRowCount();
			iface.addEndpoint( endpoint );
			
			fireTableRowsInserted( rowCount, rowCount );
		}
		
		public void removeEndpoint( int index )
		{
			String ep = getEndpointAt( index );
			iface.removeEndpoint( ep );
			fireTableRowsDeleted( index, index );
		}
		
		public void setInterface( WsdlInterface iface, DefaultEndpointStrategy strategy )
		{
			this.iface = iface;
			this.strategy = strategy;
			
			fireTableDataChanged();
		}

		public int getRowCount()
		{
			return iface == null ? 0 : iface.getEndpoints().length;
		}

		public Object getValueAt( int rowIndex, int columnIndex )
		{
			String endpoint = getEndpointAt( rowIndex );
			EndpointDefaults defaults = strategy.getEndpointDefaults( endpoint );

			switch( columnIndex )
			{
			case 0:
				return endpoint;
			case 1:
				return defaults.getUsername();
			case 2:
				return defaults.getPassword();
			case 3:
				return defaults.getDomain();
			case 4:
				return defaults.getWssType();
			case 5:
				return defaults.getWssTimeToLive();
			}

			return null;
		}

		@Override
		public boolean isCellEditable( int rowIndex, int columnIndex )
		{
			return true;
		}

		@Override
		public void setValueAt( Object aValue, int rowIndex, int columnIndex )
		{
			String endpoint = getEndpointAt( rowIndex );
			EndpointDefaults defaults = strategy.getEndpointDefaults( endpoint );
			
			if( aValue == null )
				aValue = "";
			
			switch( columnIndex )
			{
				case 0 :
				{
					iface.changeEndpoint( endpoint, aValue.toString() );
					break;
				}
				case 1 :
				{
					defaults.setUsername( aValue.toString() );
					break;
				}
				case 2 :
				{
					defaults.setPassword( aValue.toString() );
					break;
				}
				case 3 :
				{
					defaults.setDomain( aValue.toString() );
					break;
				}
				case 4 :
				{
					defaults.setWssType( aValue.toString() );
					break;
				}
				case 5 :
				{
					defaults.setWssTimeToLive( aValue.toString() );
					break;
				}
			}
		}
	}
}
